/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile;

import com.tomwheeler.example.tdlfile.impl.BasicTodoList;
import com.tomwheeler.example.tdlfile.intf.ListContentListener;
import com.tomwheeler.example.tdlfile.intf.TodoList;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.swing.event.ChangeEvent;
import org.openide.ErrorManager;
import org.openide.cookies.EditCookie;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.nodes.Node;
import org.openide.util.lookup.InstanceContent;

/**
 * <p>
 * This class manages access to the data that is in the TodoList file on disk.
 * It is non-visual; the TodoListFileDataNode represents it in the NetBeans
 * user interface.  However, it handles most of the work, including loading
 * the data from the file, tracking changes, saving it back to the file and 
 * hooking up the editor and viewer for the data.
 * </p>
 *
 * <p>
 * Much of it was generated by the File Type wizard in the NetBeans 5.0 IDE, 
 * but I had to add a lot of code specific to my needs.
 * </p>
 * 
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class TodoListFileDataObject extends MultiDataObject {
    
    private BasicTodoList todoList;
    private InstanceContent ic;
    private SaveCookie saveCookie;
    private EditCookie editCookie;
    
    public TodoListFileDataObject(FileObject pf, TodoListFileDataLoader loader)
        throws DataObjectExistsException, IOException {
        
        super(pf, loader);
        
        // InstanceContent is used to workaround the lack of a getLookup() method
        // in the DataObject class.  It's passed to AbstractLookup in the node's
        // constructor to create a lookup in the node that we can modify from here.
        // See commentary in createNodeDelegate() for more info.
        ic = new InstanceContent();
        
        // the lookup should contain the DataObject or certain things, such as
        // creating a file of this type from a template, won't work.
        ic.add(this);
        
        // create a TodoList from the file's data and make sure it will be in the node's lookup
        InputStream data = getPrimaryFile().getInputStream();
        todoList = new BasicTodoList(data);
        ic.add(todoList);
        
        // make sure that the file gets marked as modified whenever new items are
        // added to the list, or old items removed from it.  
        todoList.addListContentListener(new ListContentListener() {
            public void itemAdded(ChangeEvent e) {
                setModified(true);
            }
            public void itemRemoved(ChangeEvent e) {
                setModified(true);
            }
        });

        // add the edit cookie to the lookup here because the view action is always enabled
        editCookie = new Edit();
        ic.add(editCookie);
        
        // save is handled differently because it is conditionally available; see setModified()
        saveCookie = new Save();
    }
    
    /** intended to be used only by TodoListFileDataNode */
    TodoList getTodoList() {
        return todoList;
    }
    
    /** 
     * Marks the file as changed.  This does not, by itself, enable the Save action.
     * However, this is handled based on the way I coded it.
     */
    public void setModified(boolean isModified) {
        // Using instance cookie instead of CookieSet to work around issue #73306
        if (isModified()) {
            ic.add(saveCookie);
        } else {
            ic.remove(saveCookie);
        }
        super.setModified(isModified);
    }
    
    public Node.Cookie getCookie(Class clazz) {
        // EditCookie and ViewCookie are available unconditionally, but SaveCookie
        // is only available when this data object is modified.  The "if" statement
        // shown directly below is a workaround for the bug reported in issue #73306.
        // see the getCookie(Class) implementation in Tom Wheeler's DogFile example 
        // for what it would otherwise look like.
        if (clazz == SaveCookie.class) {
            if (isModified()) {
                return saveCookie;
            }

            // return null here so we're sure that nothing else could return the 
            // SaveCookie for this DataObject.  If we didn't, we might wind up with 
            // the problem described in issue #68285 (which is not actually a bug).
            return null;
        }

        // these cookies are available unconditionally; SaveCookie
        // is handled in setModified(boolean)
        if (clazz == EditCookie.class) {
            return editCookie;
        }
        
        return super.getCookie(clazz);
    }
    
    protected Node createNodeDelegate() {
        // we need to provide the TodoList item in the lookup so other things,
        // like the AddItemAction class, can work with it.  However, the DataObject
        // does not yet support lookups directly (see issue #62707), so we have to
        // workaround by using InstanceContent, which is a grab bag of things that
        // can be used to create an AbstractLookup in the node's constructor.
        // it would seem easier to override getLookup() in the node, but that method
        // is marked final, so this is the only way.  Note that this approach also
        // exposes some problems (see bug 73306 for one example).
        return new TodoListFileDataNode(this, ic);
    }
    
    private class Save implements SaveCookie {
        // this just writes the contents of our file out to disk.  It's 
        // called whenever the Save action is chosen (for example, from the 
        // file menu or by pressing the corresponding keyboard shortcut)
        public void save() {
            FileLock lock = null;
            try {
                FileObject fileObj = getPrimaryFile();
                lock = fileObj.lock();
                OutputStream os = fileObj.getOutputStream(lock);
                
                todoList.save(os);
            } catch (IOException ioe) {
                ErrorManager.getDefault().notify(ErrorManager.WARNING, ioe);
            } finally {
                if (lock != null) {
                    lock.releaseLock();
                }
            }

            // we're done saving.  This will remove the save cookie from the 
            // node's lookup (based on how I coded it), so the save menu item
            // will no longer be enabled (until the DataObject is modified again).
            setModified(false);
        }
    }

    /** called whenever the edit action is called (e.g. from the context menu) */
    private class Edit implements EditCookie {
        public void edit() {
            TodoListEditor editor = TodoListEditor.findInstance();
            
            // This editor just uses the NetBeans built-in TreeTableView to 
            // edit the entire list at once.  For this we just need to give it
            // a node and it takes of the rest.  But, if you wanted to create 
            // (or reuse) your own editor component that worked on a TodoList
            // instead of a node, then you'd just need to get the TodoList from
            // the node's lookup and then pass it to your editor.

            // get the node representing the whole TodoList
            Node rootNode = getNodeDelegate();
            assert(rootNode != null);
            // and tell the editor this is the node we're editing
            editor.setRootNode(rootNode);
            
            // get the array of nodes representing all the items in the list,
            // and associate those nodes with it so that, among other things,
            // we can save the changes when the editor has focus instead of
            // having to first select the node.
            Node[] itemNodes = rootNode.getChildren().getNodes();
            editor.setActivatedNodes(itemNodes);
            
            editor.open();
            editor.requestActive();
        }
    }
}