/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Tom Wheeler. Portions Copyright 2006 Tom Wheeler. All Rights 
 * Reserved. Any future updates to this code will be made available at 
 * http://www.tomwheeler.com/netbeans/
 */

package com.tomwheeler.example.tdlfile.impl;

import com.tomwheeler.example.tdlfile.intf.ListContentListener;
import com.tomwheeler.example.tdlfile.intf.TodoList;
import com.tomwheeler.example.tdlfile.intf.TodoListItem;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.swing.event.ChangeEvent;

/**
 * <p>
 * Basic implementation of the TodoList interface; it models a TodoList, which 
 * is a collection of TodoListItems.  
 * </p>
 *
 * @author <a href="http://www.tomwheeler.com/netbeans/">Tom Wheeler</a>
 */
public class BasicTodoList implements TodoList {

    private static final String NEWLINE = System.getProperty("line.separator");

    private enum ItemChangeType { ITEM_ADDED, ITEM_REMOVED };
    
    private transient Vector contentListeners;
    private List<TodoListItem> items;

    public BasicTodoList(InputStream data) {
        contentListeners = new Vector();
        items = new ArrayList<TodoListItem>();

        load(data);
    }
    
    public Iterator getItems() {
        return Collections.unmodifiableCollection(items).iterator();
    }

    public TodoListItem addItem() {
        TodoListItem item = new BasicTodoListItem(this);
        items.add(item);
        notifyListeners(new ChangeEvent("added new item"), ItemChangeType.ITEM_ADDED);
        return item;
    }

    public void removeItem(TodoListItem item) {
        items.remove(item);
        String evtText = "removed item with name '" + item.getName() + "'";
        notifyListeners(new ChangeEvent(evtText), ItemChangeType.ITEM_REMOVED);
    }

    private void load(InputStream data) {
        // parse the underlying file to get the initial data
        try {
            BufferedInputStream bis = new BufferedInputStream(data);
            BufferedReader reader = new BufferedReader(new InputStreamReader(bis));

            // clear list, updating all listeners in the process
            for (TodoListItem item: items) {
                this.removeItem(item);
            }

            String thisLine;
            while ((thisLine = reader.readLine()) != null) {
                String[] fields = thisLine.split(",");
                String name = fields[0];
                String desc = fields[1];
                String dateStr = fields[2];
                Date dueDate = new Date(Long.valueOf(dateStr));

                BasicTodoListItem item = (BasicTodoListItem)addItem();
                item.setName(name);
                item.setDesc(desc);
                item.setDueDate(dueDate);
            }
        } catch (IOException ioe) {
            ioe.printStackTrace();
        }
    }
    
    public void save(OutputStream os) {
        PrintWriter pw = new PrintWriter(os);

        for (Iterator it = getItems(); it.hasNext();) {
            TodoListItem item = (TodoListItem) it.next();
            pw.write(item.getName());
            pw.write(",");
            pw.write(item.getDesc());
            pw.write(",");
            String dateLongStr = String.valueOf(item.getDueDate().getTime());
            pw.write(dateLongStr);
            if (it.hasNext()) {
                pw.write(NEWLINE);
            }                
        }

        pw.flush();
        pw.close();        
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        
        if (! (o instanceof BasicTodoList)) {
            return false;
        }
        
        BasicTodoList that = (BasicTodoList)o;
        if (this.items.equals(that.items)) {
            return true;
        }
        
        return false;
    }
    
    public int hashCode() {
        return items.hashCode();
    }
    
    public String toString() {
        return "BasicTodoList: [ " + items.size() + " items ]";
    }

    public void addListContentListener(ListContentListener listener) {
        contentListeners.add(listener);
    }
    
    public void removeListContentListener(ListContentListener listener) {
        contentListeners.remove(listener);
    }
    
    /** convenience method to create a new event and notify registered listeners */
    private void notifyListeners(final ChangeEvent event, ItemChangeType eventType) {
        Collection listenersClone = (Collection)contentListeners.clone();
        for (Iterator iter = listenersClone.iterator(); iter.hasNext();) {
            ListContentListener listener = (ListContentListener) iter.next();
            if ( ItemChangeType.ITEM_ADDED.equals(eventType)) {
                listener.itemAdded(event);
            } else if ( ItemChangeType.ITEM_REMOVED.equals(eventType )) {
                listener.itemRemoved(event);
            }
        }
    }
}
